#.rst:
# FindARToolKit
# --------
#
# Find ARToolkit
#
# Find the artoolkit libraries (AR)
#
# ::
#
#   This module defines the following variables:
#      ARTOOLKIT_FOUND        - True if ARTOOLKIT_INCLUDE_DIR & ARTOOLKIT_LIBRARY are found
#      ARTOOLKIT_LIBRARIES    - Set when ARTOOLKIT_LIBRARY is found
#      ARTOOLKIT_INCLUDE_DIRS - Set when ARTOOLKIT_INCLUDE_DIR is found
#
#
# ::
#
#      ARTOOLKIT_INCLUDE_DIR - where to find arConfig.h, etc.
#      ARTOOLKIT_LIBRARY     - the artoolkit library
#      ARTOOLKIT_VERSION_STRING - the version of artoolkit found (since CMake 2.8.8)


find_path(ARTOOLKIT_INCLUDE_DIR NAMES AR/arConfig.h
          DOC "The ARToolKit (arConfig) include directory"
)

find_library(ARTOOLKIT_LIBRARY NAMES AR
          DOC "The ARToolKit (AR) library"
)

if(ARTOOLKIT_INCLUDE_DIR AND EXISTS "${ARTOOLKIT_INCLUDE_DIR}/AR/config.h")
  file(STRINGS "${ARTOOLKIT_INCLUDE_DIR}/AR/config.h" artoolkit_version_str REGEX "^#define[\t ]+AR_HEADER_VERSION_STRING[\t ]+\".*\"")

  string(REGEX REPLACE "^.*AR_HEADER_VERSION_STRING[\t ]+\"([^\"]*)\".*$" "\\1" ARTOOLKIT_VERSION_STRING "${artoolkit_version_str}")
  unset(artoolkit_version_str)
endif()

# handle the QUIETLY and REQUIRED arguments and set ARTOOLKIT_FOUND to TRUE if
# all listed variables are TRUE
include(${CMAKE_CURRENT_LIST_DIR}/FindPackageHandleStandardArgs.cmake)
FIND_PACKAGE_HANDLE_STANDARD_ARGS(ARTOOLKIT
                                  REQUIRED_VARS ARTOOLKIT_LIBRARY ARTOOLKIT_INCLUDE_DIR
                                  VERSION_VAR ARTOOLKIT_VERSION_STRING)

if(ARTOOLKIT_FOUND)
  set( ARTOOLKIT_LIBRARIES ${ARTOOLKIT_LIBRARY} )
  set( ARTOOLKIT_INCLUDE_DIRS ${ARTOOLKIT_INCLUDE_DIR} )
endif()

mark_as_advanced(ARTOOLKIT_INCLUDE_DIR ARTOOLKIT_LIBRARY)
